define([
    'testsuite/qunit',
    'testsuite/qunit-fixture!de_epages/contextualappads/ui/appsdialog-test',
    'testsuite/sinon',
    'jquery',
    'de_epages/contextualappads/ui/appsdialog',
    'ep',

    '$ready!'
], function (QUnit, fixture, sinon, $, de_epages, ep) {

    var mockApps = [{
            appUrl: 'https://www.billbee.de',
            description: 'Billbee bietet eine umfangreiche aber einfach zu bedienende...',
            id: '82F2085D-A912-4BA8-B0D8-B03713593906',
            detailId: '82F2085D-A912-4BA8-B0D8-B03713593906',
            name: 'Billbee',
            installed: false,
            logoUrl: 'bild1.jpg'
        }, {
            appUrl: 'https://www.facebook.de',
            description: 'Billbee bietet eine umfangreiche aber einfach zu bedienende...',
            id: '2',
            detailId: '2',
            name: 'Facebook',
            installed: false,
            logoUrl: 'bild2.jpg'
        }, {
            appUrl: 'https://www.whatsapp.de',
            description: 'Billbee bietet eine umfangreiche aber einfach zu bedienende...',
            id: '3',
            detailId: '3',
            name: 'WhatsApp',
            installed: false,
            logoUrl: 'bild3.jpg'
        }]

    QUnit.module('de_epages.contextualappads.ui.appsdialog', {
        setup: fixture.init,
        teardown: fixture.destroy
    });

    QUnit.asyncTest("widget test presence of methods", function () {

        $.each([
            '_create',
            '_createDialog',
            '_openInstallationDialog',
            '_addAppsToContext',
            '_toggleButton',
            '_initSlider',
            '_setArrows',
            '_getPosition',
            'open',
            'close',
            'destroy'
        ], function (i, fn) {
                QUnit.equal(
                $.isFunction($.ui.contextualappadsUiAppsdialog.prototype[fn]),
                true,
                '$.isFunction(' + fn + '):'
            );
        });

        QUnit.start();
    });

    QUnit.asyncTest("widget ui tests", function () {

        /* START - fake ajax request */
        ep.ajax = function (o) {
            var result = $.Deferred(),
                resolveSend,
                seenAppData = {};

            // build data object to compare sent data with
            seenAppData[mboContext] = mockApps.map(function(app) { return app.id; });

            if (o.data && o.data.ChangeAction) {
                if (o.data.ChangeAction === "JSONSaveSeenAppIDs") {
                    // test if sent data are correct
                    QUnit.ok(
                        o.data.SeenAppIDs === JSON.stringify(seenAppData),
                        'Correct context and app ids will be sent.'
                    );

                    result.resolve({
                        success: true
                    });
                }
            } else {
                result.reject();
            }

            return result;
        };
        /* END - fake ajax request */

        var $div = $('.ep-top-bar-buttons-context-apps'),
            $buttonNumberElement = $('.ep-top-bar-apps-count'),
            mboContext = 'product',
            resizeSpy,
            toggleSpy,
            appLinks,
            appImgs,
            regExp,
            i;

        $div.contextualappadsUiAppsdialog({
            apps: mockApps,
            shopId: 15,
            showNumberBubble: true,
            mboContext: mboContext
        });

        setTimeout(function() {
            QUnit.ok(
                $div.data('uiContextualappadsUiAppsdialog') !== undefined,
                'Widget exists'
            );

            // register spy on _getPosition method => number of executions can be investigated
            resizeSpy = sinon.spy($div.data('uiContextualappadsUiAppsdialog'), '_getPosition');
            // register spy on _toggleButton method => number of executions can be investigated
            toggleSpy = sinon.spy($div.data('uiContextualappadsUiAppsdialog'), '_toggleButton');

            QUnit.ok(
                $div.data('uiContextualappadsUiAppsdialog').dialog === undefined,
                'Slider dialog instance does not exist'
            );

            // test if element for unseen apps is visible on start up
            QUnit.ok(
                !$div.hasClass('ep-top-bar-apps-count-invisible'),
                'Element for number of unseen apps is visible'
            );

            // Trigger click-event to open dialog
            $div.trigger('click');

            setTimeout(function() {
                QUnit.ok(
                    $div.data('uiContextualappadsUiAppsdialog').dialog !== undefined,
                    'Slider dialog instance exists'
                );
                QUnit.ok(
                    $div.data('uiContextualappadsUiAppsdialog').dialog.is(':visible') ,
                    'Slider dialog is visible'
                );
                QUnit.equal(
                    $div.data('uiContextualappadsUiAppsdialog').dialog.find('.ep-appdialog-slider-wrapper .ep-appdialog-slider-teaser').length,
                    mockApps.length,
                    'Slider dialog contains ' + mockApps.length + ' Apps'
                );

                QUnit.ok(
                    resizeSpy.calledTwice,
                    'Slider dialogs position was calculated.'
                );

                // test if element for unseen apps is invisible after click on button
                QUnit.ok(
                    $div.hasClass('ep-top-bar-apps-count-invisible'),
                    'Element for number of unseen apps is now invisible'
                );

                appLinks = $div.data('uiContextualappadsUiAppsdialog').dialog.find('.ep-appdialog-slider-wrapper .ep-appdialog-slider-teaser-link');
                appImgs =  $div.data('uiContextualappadsUiAppsdialog').dialog.find('.ep-appdialog-slider-wrapper img');

                mockApps.forEach( function(app, index) {
                    QUnit.equal(
                        appLinks.eq(index).attr('title'),
                        app.name,
                        'Slider dialog contains the following app name ' + app.name
                    );
                    QUnit.ok(
                        RegExp(app.logoUrl.replace(/\./g, "\\.")).test(appImgs.eq(index).attr('src')),
                        'Slider dialog contains the following app image ' + app.logoUrl
                    );
                })

                QUnit.equal(
                    $div.data('uiContextualappadsUiAppsdialog').dialog.find('.ep-appdialog-slider-wrapper .ep-appdialog-slider-teaser').length,
                    $div.data('uiContextualappadsUiAppsdialog').apps.length,
                    'Slider dialog contains ' + $div.data('uiContextualappadsUiAppsdialog').apps.length + ' App(s)'
                );

                // Close dialog
                $div.contextualappadsUiAppsdialog('close');

                setTimeout(function() {
                    QUnit.ok(
                        !$div.data('uiContextualappadsUiAppsdialog').dialog.is(':visible') ,
                        'Slider dialog is closed'
                    );

                    // Open dialog
                    $div.contextualappadsUiAppsdialog('open');
                    $(window).trigger('resize.contextualappadsUiAppsdialog')

                    setTimeout(function() {
                        QUnit.ok(
                            $div.data('uiContextualappadsUiAppsdialog').dialog.is(':visible') ,
                            'Slider dialog is open'
                        );

                        // test if _getPosition was called again after resize event
                        QUnit.ok(
                            resizeSpy.calledThrice,
                            'Slider dialogs position was recalculated after window resize.'
                        );

                        // open installation dialog
                        $('.ep-appdialog-slider-teaser-readmore:first').trigger('click');

                        setTimeout(function() {
                            // test visibilities of slider and detail dialogs
                            QUnit.ok(
                                $div.data('uiContextualappadsUiAppsdialog').dialog.is(':visible') === false,
                                'Slider dialog is closed'
                            );
                            QUnit.ok(
                                $('.ep-appdialog-detailsdialog').is(':visible'),
                                'Detail dialog is open'
                            );

                            // test UI of detail dialog
                            QUnit.ok(
                                $('.ep-appdialog-detailsdialog').find('.ep-button-close').length,
                                'Detail dialog contains a "cancel" button'
                            );
                            QUnit.ok(
                                $('.ep-appdialog-detailsdialog').find('.ep-button-app-install').length,
                                'Detail dialog contains an "install" button'
                            );

                            // test if onAfterIframeLoaded callback
                            QUnit.ok(
                                toggleSpy.calledOnce,
                                '"_toggleButton" method was called from the "onAfterIframeLoaded" callback.'
                            );
                            QUnit.ok(
                                toggleSpy.args[0][1] === false,
                                '"_toggleButton" method was called with parameter "' + toggleSpy.args[0][1] + '" to enable install button'
                            )

                            // test correct app detailID in iframe source
                            regExp = new RegExp('appID=' + mockApps[0].detailId + '&');
                            QUnit.ok(
                                regExp.test($('.ep-appdialog-detailsdialog').find('iframe')[0].src),
                                'iFrame source (src) contains the correct app detailID "' + mockApps[0].detailId +'".'
                            );

                            // close detail dialog again
                            $('.ep-appdialog-detailsdialog').find('.ep-button-close').trigger('click');

                            setTimeout(function() {
                                QUnit.ok(
                                    $div.data('uiContextualappadsUiAppsdialog').dialog.is(':visible'),
                                    'Slider dialog is open'
                                );
                                QUnit.ok(
                                    $('.ep-appdialog-detailsdialog').is(':visible') === false,
                                    'Details dialog is closed'
                                );

                                // Destroy widget
                                $div.contextualappadsUiAppsdialog('destroy');

                                setTimeout(function() {
                                    QUnit.ok(
                                        $div.data('uiContextualappadsUiAppsdialog') === undefined,
                                        'Widget destroyed'
                                    );
                                    QUnit.ok(
                                        !$('.ui-dialog').length,
                                        'All dialog elements were removed from DOM'
                                    );

                                    // restore spies after tests
                                    resizeSpy.restore();
                                    toggleSpy.restore();

                                    // start async tests
                                    QUnit.start();
                                }, 300);
                            }, 300);
                        }, 200);
                    }, 200);
                }, 200);
            }, 400);
        }, 500);
    });
});